@@ -1,7 +1,8 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import csv
-from io import BytesIO
+import sys
+from io import BytesIO, StringIO
 
 from sql import Table
 
@@ -93,16 +94,33 @@ class AccountFrFEC(Wizard):
             ])
 
     def transition_generate(self):
-        fec = BytesIO()
+        if sys.version_info < (3,):
+            fec = BytesIO()
+        else:
+            fec = StringIO()
         writer = self.get_writer(fec)
+        writer.writerow(self.get_header())
         format_date = self.get_format_date()
         format_number = self.get_format_number()
+
+        def replace_delimiter(c):
+            delimiter = writer.dialect.delimiter
+            return (c or '').replace(delimiter, ' ')
+        if sys.version_info < (3,):
+            def convert(c):
+                return replace_delimiter(c).encode('utf-8')
+        else:
+            convert = replace_delimiter
+
         for row in self.get_start_balance():
-            writer.writerow([(c or '').encode('utf-8') for c in row])
+            writer.writerow(map(convert, row))
         for line in self.get_lines():
             row = self.get_row(line, format_date, format_number)
-            writer.writerow([(c or '').encode('utf-8') for c in row])
-        self.result.file = fec.getvalue()
+            writer.writerow(map(convert, row))
+        value = fec.getvalue()
+        if not isinstance(value, bytes):
+            value = value.encode('utf-8')
+        self.result.file = self.result.__class__.file.cast(value)
         return 'result'
 
     def default_result(self, fields):
@@ -119,20 +137,47 @@ class AccountFrFEC(Wizard):
             }
 
     def get_writer(self, fd):
-        return csv.writer(fd)
+        return csv.writer(
+            fd, delimiter='\t', doublequote=False, escapechar='\\',
+            quoting=csv.QUOTE_NONE)
 
     def get_format_date(self):
         pool = Pool()
         Lang = pool.get('ir.lang')
-        fr, = Lang.search([('code', '=', 'fr_FR')])
-        return lambda value: Lang.strftime(value, fr.code, '%Y%m%d')
+        return lambda value: Lang.strftime(value, 'fr_FR', '%Y%m%d')
 
     def get_format_number(self):
         pool = Pool()
         Lang = pool.get('ir.lang')
-        fr, = Lang.search([('code', '=', 'fr_FR')])
+        fr = Lang(
+            decimal_point=',',
+            thousands_sep='',
+            grouping='[]',
+            )
         return lambda value: Lang.format(fr, '%.2f', value)
 
+    def get_header(self):
+        return [
+            'JournalCode ',
+            'JournalLib ',
+            'EcritureNum',
+            'EcritureDate',
+            'CompteNum',
+            'CompteLib',
+            'CompAuxNum',
+            'CompAuxLib',
+            'PieceRef',
+            'PieceDate',
+            'EcritureLib',
+            'Debit',
+            'Credit',
+            'EcritureLet',
+            'DateLet',
+            'ValidDate',
+            'Montantdevise',
+            'Idevise',
+            ]
+
     def get_start_balance(self):
         pool = Pool()
         Account = pool.get('account.account')
@@ -147,10 +192,15 @@ class AccountFrFEC(Wizard):
             accounts = Account.search([])
 
         for account in accounts:
-            if not account.credit and not account.debit:
+            if account.credit == account.debit:
                 continue
+            if account.debit > account.credit:
+                debit, credit = account.debit - account.credit, 0
+            else:
+                debit, credit = 0, account.debit - account.credit
             yield [
-                self.start.deferral_journal.code,
+                self.start.deferral_journal.code
+                or self.start.deferral_journal.name,
                 self.start.deferral_journal.name,
                 self.start.deferral_post_number,
                 format_date(self.start.fiscalyear.start_date),
@@ -158,11 +208,11 @@ class AccountFrFEC(Wizard):
                 account.name,
                 '',
                 '',
-                '',
+                '-',
                 format_date(self.start.fiscalyear.start_date),
-                '',
-                format_number(account.debit or 0),
-                format_number(account.credit or 0),
+                '-',
+                format_number(debit),
+                format_number(credit),
                 '',
                 '',
                 format_date(self.start.fiscalyear.start_date),
@@ -195,7 +245,7 @@ class AccountFrFEC(Wizard):
         if line.reconciliation and line.reconciliation.date <= end_date:
             reconciliation = line.reconciliation
         return [
-            line.move.journal.code,
+            line.move.journal.code or line.move.journal.name,
             line.move.journal.name,
             line.move.post_number,
             format_date(line.move.date),
@@ -203,9 +253,9 @@ class AccountFrFEC(Wizard):
             line.account.name,
             line.party.code if line.party else '',
             line.party.name if line.party else '',
-            self.get_reference(line),
+            self.get_reference(line) or '-',
             format_date(self.get_reference_date(line)),
-            description(),
+            description() or '-',
             format_number(line.debit or 0),
             format_number(line.credit or 0),
             reconciliation.rec_name if reconciliation else '',
